package com.abi.qms.platform.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.abi.qms.platform.dao.entity.UserInfo;
import com.abi.qms.platform.dao.entity.UserMappingInfoRole;
import com.abi.qms.platform.dao.entity.UserRole;
import com.abi.qms.platform.dao.mapper.UserInfoMapper;
import com.abi.qms.platform.dao.mapper.UserMappingInfoRoleMapper;
import com.abi.qms.platform.dao.mapper.UserRoleMapper;
import com.abi.qms.platform.dto.req.BatchEnableUserReq;
import com.abi.qms.platform.dto.req.CreateUserReq;
import com.abi.qms.platform.dto.req.ListUserReq;
import com.abi.qms.platform.dto.req.UpdateUserReq;
import com.abi.qms.platform.dto.res.GetUserInfoRes;
import com.abi.qms.platform.dto.res.ListRoleTreeRes;
import com.abi.qms.platform.dto.res.ListUserRes;
import com.abi.qms.platform.infrastructure.util.PageUtil;
import com.abi.qms.platform.infrastructure.util.UserUtil;
import com.abi.qms.platform.service.RoleManagerService;
import com.abi.qms.platform.service.UserManagerService;
import com.abi.task.common.api.exception.BusinessException;
import com.abi.task.common.utils.PojoConverterUtils;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 用户管理
 * </p>
 *
 * @author Andy.Tan
 */
@Slf4j
@Service
public class UserManagerServiceImpl extends ServiceImpl<UserInfoMapper, UserInfo> implements UserManagerService {

    @Autowired
    private UserInfoMapper userInfoMapper;

    @Autowired
    private UserRoleMapper userRoleMapper;

    @Autowired
    private UserMappingInfoRoleMapper userMappingInfoRoleMapper;

    @Autowired
    private RoleManagerService roleManagerService;

    @Autowired
    private UserUtil userUtil;

    /**
     * 创建用户
     **/
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void createUser(CreateUserReq createUserReq) {

        QueryWrapper<UserRole> roleQueryWrapper = new QueryWrapper<>();
        roleQueryWrapper.eq("role_code", createUserReq.getRoleCode());
        roleQueryWrapper.eq("is_delete", 0);
        UserRole userRole = userRoleMapper.selectOne(roleQueryWrapper);
        if (Objects.isNull(userRole)) {
            throw new BusinessException("该角色不存在。");
        }
        if (lambdaQuery().eq(UserInfo::getEmployeeNo, createUserReq.getEmployeeNo().trim()).count() > 0) {
            throw new BusinessException("工号" + createUserReq.getEmployeeNo() + "已存在，请勿重复创建。");
        }
        if (lambdaQuery().eq(UserInfo::getUserName, createUserReq.getUserName().trim()).count() > 0) {
            throw new BusinessException("用户" + createUserReq.getUserName() + "已存在，请勿重复创建。");
        }
        UserInfo userInfo = new UserInfo();
        userInfo.setUserName(createUserReq.getUserName());
        userInfo.setEmployeeNo(createUserReq.getEmployeeNo());
        userInfo.setEmail(createUserReq.getEmail());
        userInfo.setPassword(createUserReq.getPassword());
        userInfo.setIsAbi(createUserReq.getIsAbi());
        userInfo.setFactoryId(createUserReq.getFactoryId());
        userInfo.setRoleCode(createUserReq.getRoleCode());

        UserInfo loginUser = userUtil.getUser();
        userInfo.setCreateBy(loginUser.getId());
        save(userInfo);
        userInfo.setLinkFlag(loginUser.getLinkFlag() + userInfo.getId() + ",");
        updateById(userInfo);
        //添加用户和角色的映射关系
        UserMappingInfoRole userMappingInfoRole = new UserMappingInfoRole();
        userMappingInfoRole.setUserId(userInfo.getId());
        userMappingInfoRole.setRoleCode(userRole.getRoleCode());
        userMappingInfoRole.setRoleName(userRole.getRoleName());
        userMappingInfoRoleMapper.insert(userMappingInfoRole);
    }

    /**
     * 修改用户
     **/
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateUser(UpdateUserReq updateUserReq) {

        QueryWrapper<UserRole> roleQueryWrapper = new QueryWrapper<>();
        roleQueryWrapper.eq("role_code", updateUserReq.getRoleCode());
        roleQueryWrapper.eq("is_delete", 0);
        UserRole userRole = userRoleMapper.selectOne(roleQueryWrapper);
        if (Objects.isNull(userRole)) {
            throw new BusinessException("该角色不存在。");
        }
        UserInfo userInfo = userInfoMapper.selectById(updateUserReq.getId());
        if(Objects.isNull(userInfo)){
            throw new BusinessException("该用户不存在。");
        }
        userInfo.setEmail(updateUserReq.getEmail());
        userInfo.setFactoryId(updateUserReq.getFactoryId());
        userInfo.setRoleCode(updateUserReq.getRoleCode());
        userInfoMapper.updateById(userInfo);
        //修改用户和角色的映射关系
        if(!userRole.getRoleCode().equals(updateUserReq.getRoleCode())) {
            QueryWrapper<UserMappingInfoRole> qwUR = new QueryWrapper<>();
            qwUR.eq("user_id", userInfo.getId());
            qwUR.eq("is_delete", 0);
            UserMappingInfoRole userMappingInfoRole = userMappingInfoRoleMapper.selectOne(qwUR);
            if (Objects.nonNull(userMappingInfoRole)) {
                userMappingInfoRole.setRoleCode(userRole.getRoleCode());
                userMappingInfoRole.setRoleName(userRole.getRoleName());
                userMappingInfoRoleMapper.updateById(userMappingInfoRole);
            }
        }
    }

    /**
     * 查询角色树
     **/
    @Override
    public ListRoleTreeRes listRoleTree() {
        //获取当前登录的用户
        UserInfo user = userUtil.getUser();
        //获取当前用户的角色
        UserRole role = roleManagerService.getRoleByCode(user.getRoleCode());

        //1-查询当前角色以及其下的所有角色
        List<UserRole> userRoleList = userRoleMapper.listRoleTreeEnable(role);

        //构造出参
        List<ListRoleTreeRes.RoleBean> roleBeanList = PojoConverterUtils.copyList(userRoleList, ListRoleTreeRes.RoleBean.class);
        ListRoleTreeRes res = new ListRoleTreeRes();
        res.setRoleBeanList(roleBeanList);

        return res;
    }


    /**
     * 查询单个用户信息
     **/
    @Override
    public GetUserInfoRes getUserInfo(Long userInfoId) {
        UserInfo userInfo = userInfoMapper.selectById(userInfoId);
        GetUserInfoRes getUserInfoRes = new GetUserInfoRes();
        BeanUtils.copyProperties(userInfo, getUserInfoRes);
        return getUserInfoRes;
    }

    /**
     * 查询用户列表
     **/
    @Override
    public ListUserRes listUser(ListUserReq listUserReq) {
        UserInfo user = userUtil.getUser();
        listUserReq.setLinkFlag("," + user.getId() + ",");
        IPage iPage = userInfoMapper.selectUserInfoPage(PageUtil.createPage(listUserReq), listUserReq);
        List<UserInfo> records = iPage.getRecords();
        //封装出参
        ListUserRes res = new ListUserRes();
        //放入分页信息
        PageUtil.copyPageInfo(res, iPage);
        //放入出参列表
        List<ListUserRes.UserInfoBean> eventBeanList = PojoConverterUtils.copyList(records, ListUserRes.UserInfoBean.class);
        res.setUserInfoBeanList(eventBeanList);
        return res;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void enableUser(Long userInfoId) {
        UserInfo userInfo = userInfoMapper.selectById(userInfoId);
        if (ObjectUtil.isNotEmpty(userInfo)) {
            userInfo.setIsEnable(1 - userInfo.getIsEnable());
            userInfoMapper.updateById(userInfo);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void batchEnableUser(BatchEnableUserReq req) throws Exception {
        userInfoMapper.batchEnableUser(req);
    }


}
